/**************************************************************************
 *
 * FILE  cbm_mem.h
 * Copyright (c) 2007, 2012, 2018 Daniel Kahlin
 * Written by Daniel Kahlin <daniel@kahlin.net>
 *
 * DESCRIPTION
 *   cbm memory handling
 *
 ******/
#ifndef CBM_MEM_H
#define CBM_MEM_H

#include <stdint.h>

#define RAM_SIZE 0x10000

typedef uint32_t cbm_ptr_t;
#define CBM_PTR_UNDEFINED (0xffffffffUL)

int is_defined_addr(cbm_ptr_t ad);
cbm_ptr_t min_addr(cbm_ptr_t ad1, cbm_ptr_t ad2);
cbm_ptr_t max_addr(cbm_ptr_t ad1, cbm_ptr_t ad2);

typedef struct {
    cbm_ptr_t sa;
    cbm_ptr_t ea;
} cbm_range_t;
#define CBM_RANGE_UNDEFINED (cbm_range_t){ CBM_PTR_UNDEFINED, CBM_PTR_UNDEFINED }

int is_defined_range(cbm_range_t rng);
void update_range(cbm_range_t *rng, cbm_ptr_t ad);
cbm_range_t max_range(cbm_range_t rng1, cbm_range_t rng2);

typedef struct cbm_mem {
    uint8_t *ram;
    uint8_t *usemap;
    int size;
} cbm_mem_t;

enum mode_t {
    MODE_NORMAL = 0,
    MODE_NEWADDR,
    MODE_RAW
};

cbm_mem_t *create_mem(void);
void destroy_mem(cbm_mem_t *cm);
void clear_mem(cbm_mem_t *cm, cbm_ptr_t sa, cbm_ptr_t ea);
uint8_t get_byte(cbm_mem_t *cm, cbm_ptr_t ad);
uint8_t get_byte_defined(cbm_mem_t *cm, cbm_ptr_t ad);
void set_byte(cbm_mem_t *cm, cbm_ptr_t ad, uint8_t val);
void unset_byte(cbm_mem_t *cm, cbm_ptr_t ad);
uint16_t get_word(cbm_mem_t *cm, cbm_ptr_t ad);
void set_word(cbm_mem_t *cm, cbm_ptr_t ad, uint16_t val);
cbm_ptr_t get_low_addr(cbm_mem_t *cm);
cbm_ptr_t get_high_addr(cbm_mem_t *cm);
cbm_range_t get_range(cbm_mem_t *cm);
void copy_mem(cbm_mem_t *cm, cbm_ptr_t sa, cbm_ptr_t ea, cbm_ptr_t da);
void copy_mem2(cbm_mem_t *scm, cbm_ptr_t sa, cbm_ptr_t ea, cbm_mem_t *dcm, cbm_ptr_t da);
void insert_mem(uint8_t *src, int len, cbm_mem_t *dcm, cbm_ptr_t da);
void load_mem(cbm_mem_t *cm, const char *name, cbm_ptr_t la, int offs, int len, enum mode_t mode, cbm_ptr_t *aptr, int *lptr);
void save_mem(cbm_mem_t *cm, const char *name, cbm_ptr_t sa, cbm_ptr_t ea, cbm_ptr_t la, enum mode_t mode);

#endif /* CBM_MEM_H */
/* eof */
