/**************************************************************************
 *
 * FILE  output.c
 * Copyright (c) 2018 Daniel Kahlin
 * Written by Daniel Kahlin <daniel@kahlin.net>
 *
 * DESCRIPTION
 *   hex output functions
 *
 ******/
#include <stdio.h>
#include <stdint.h>

#include "cbm_mem.h"
#include "global.h"
#include "output.h"

static void dump_line(Dumper *dmp, cbm_ptr_t addr, uint8_t *buf, int len);

void init_dumper(Dumper *dmp)
{
    dmp->long_addr = 0;
    dmp->max_line_len = 16;
    dmp->printable = 1;
    dmp->len = 0;
}


void flush_dumper(Dumper *dmp)
{
    if (dmp->len) {
	dump_line(dmp, dmp->addr, dmp->buf, dmp->len);
	dmp->len = 0;
    }
}


void dump_data(Dumper *dmp, cbm_ptr_t addr, uint8_t data)
{
    if (dmp->len == 0) {
	dmp->addr = addr;
    }

    dmp->buf[dmp->len++] = data;

    if (dmp->len >= dmp->max_line_len) {
	flush_dumper(dmp);
    }
}


char printable(uint8_t c)
{
    if (c > 0x00 && c <= ('Z' - 0x40)) {
	return c | 0x60;
    }
    if (c < 0x20 || c > 0x7e) {
	return '.';
    }
    return c;
}


static void dump_line(Dumper *dmp, cbm_ptr_t addr, uint8_t *buf, int len)
{
    int i;
    int offs = 0;

    if (dmp->long_addr) {
	printf("%06X ", addr);
    } else {
	printf("%04X ", addr);
    }


    for (i = 0; i < offs; i++) {
	printf("   ");
    }
    for (i = offs; i < dmp->max_line_len; i++) {
	if (i == 8) {
	    printf(" ");
	}
	if (i < len) {
	    printf(" %02X", buf[i]);
	} else {
	    printf("   ");
	}
    }

    for (i = 0; i < offs; i++) {
	printf(" ");
    }
    printf(" |");
    for (i = offs; i < len; i++) {
	printf("%c", printable(buf[i]));
    }
    printf("|");
    printf("\n");
}

/* eof */
